@@ -762,7 +762,7 @@ shares_error_quark (void)
 	static GQuark quark;
 
 	if (quark == 0)
-		quark = g_quark_from_string ("mate-file-manager-shares-error-quark"); /* not from_static_string since we are a module */
+		quark = g_quark_from_string ("caja-share-error-quark"); /* not from_static_string since we are a module */
 
 	return quark;
 }
@@ -997,9 +997,6 @@ shares_free_share_info_list (GSList *list)
 	GSList *l;
 
 	for (l = list; l; l = l->next) {
-		ShareInfo *info;
-
-		info = l->data;
 		shares_free_share_info (l->data);
 	}
 
